<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Main Taxiar Core Class
 *
 * The main class that initiates and runs the plugin.
 *
 * @since 1.0.0
 */

final class Taxiar_Extension {

	/**
	 * Plugin Version
	 *
	 * @since 1.0.0
	 *
	 * @var string The plugin version.
	 */
	const VERSION = '1.0.0';

	/**
	 * Minimum Elementor Version
	 *
	 * @since 1.0.0
	 *
	 * @var string Minimum Elementor version required to run the plugin.
	 */

	const MINIMUM_ELEMENTOR_VERSION = '2.0.0';

	/**
	 * Minimum PHP Version
	 *
	 * @since 1.0.0
	 *
	 * @var string Minimum PHP version required to run the plugin.
	 */
	const MINIMUM_PHP_VERSION = '7.0';


	/**
	 * Instance
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @static
	 *
	 * @var Elementor_Test_Extension The single instance of the class.
	 */

	private static $_instance = null;

	/**
	 * Instance
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @static
	 *
	 * @return Elementor_Test_Extension An instance of the class.
	 */
	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;

	}

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function __construct() {
		add_action( 'plugins_loaded', [ $this, 'init' ] );
	}

	/**
	 * Initialize the plugin
	 *
	 * Load the plugin only after Elementor (and other plugins) are loaded.
	 * Checks for basic plugin requirements, if one check fail don't continue,
	 * if all check have passed load the files required to run the plugin.
	 *
	 * Fired by `plugins_loaded` action hook.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function init() {

		// Check if Elementor installed and activated

		if ( ! did_action( 'elementor/loaded' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_missing_main_plugin' ] );
			return;
		}

		// Check for required Elementor version

		if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_elementor_version' ] );
			return;
		}

		// Check for required PHP version

		if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_php_version' ] );
			return;
		}


		// Add Plugin actions

		add_action( 'elementor/widgets/register', [ $this, 'init_widgets' ] );


        // Register widget scripts

		add_action( 'elementor/frontend/after_enqueue_scripts', [ $this, 'widget_scripts' ]);


		// Specific Register widget scripts

		// add_action( 'elementor/frontend/after_register_scripts', [ $this, 'taxiar_regsiter_widget_scripts' ] );
		// add_action( 'elementor/frontend/before_register_scripts', [ $this, 'taxiar_regsiter_widget_scripts' ] );


        // category register

		add_action( 'elementor/elements/categories_registered',[ $this, 'taxiar_elementor_widget_categories' ] );
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have Elementor installed or activated.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_missing_main_plugin() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor */
			esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'taxiar' ),
			'<strong>' . esc_html__( 'Taxiar Core', 'taxiar' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'taxiar' ) . '</strong>'
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required Elementor version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_elementor_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */

			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'taxiar' ),
			'<strong>' . esc_html__( 'Taxiar Core', 'taxiar' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'taxiar' ) . '</strong>',
			 self::MINIMUM_ELEMENTOR_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
	}
	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required PHP version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_php_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(

			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'taxiar' ),
			'<strong>' . esc_html__( 'Taxiar Core', 'taxiar' ) . '</strong>',
			'<strong>' . esc_html__( 'PHP', 'taxiar' ) . '</strong>',
			 self::MINIMUM_PHP_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
	}

	/**
	 * Init Widgets
	 *
	 * Include widgets files and register them
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */

	public function init_widgets() {

		$widget_register = \Elementor\Plugin::instance()->widgets_manager;

		// Header Include file & Widget Register
		require_once( TAXIAR_ADDONS . '/header/header.php' );
		$widget_register->register ( new \Taxiar_Header() );

		// Include All Widget Files
		foreach($this->Taxiar_Include_File() as $widget_file_name){
			require_once( TAXIAR_ADDONS . '/widgets/taxiar-'."$widget_file_name".'.php' );
		}

		// All Widget Register
		foreach($this->Taxiar_Register_File() as $name){
			$widget_register->register ( $name );
		}
		
	}

	public function Taxiar_Include_File(){
		return [
			'banner', 
			'section-title', 
			'button', 
			'blog', 
			'service', 
			'testimonial', 
			'team', 
			'team-info', 
			'image', 
			'contact-info', 
			'contact-form', 
			'counterup', 
			'faq', 
			'brand-logo', 
			'cta', 
			'gallery', 
			'info-box', 
			'portfolio', 
			'portfolio-info',
			'newsletter', 
			'menu-select', 
 
			'service-tab',
			'taxi-service',
			'taxi-info',
			'download-button',
			'animated-shape', 
			'arrows', 
			'tab-builder', 
			'skill', 
			'about-info', 
			'step', 
			// 'choose', 
			'features', 
			'video', 
			'price' 
		];
	}

	public function Taxiar_Register_File(){
		return [
			new \Taxiar_Banner() ,
			new \Taxiar_Section_Title(),
			new \Taxiar_Button(),
			new \Taxiar_Blog(),
			new \Taxiar_Service(),
			new \Taxiar_Testimonial(),
			new \Taxiar_Team(),
			new \Taxiar_Team_info(),
			new \Taxiar_Image(),
			new \Taxiar_Contact_Info(),
			new \Taxiar_Contact_Form(),
			new \Taxiar_Counterup(),
			new \Taxiar_Faq(),
			new \Taxiar_Brand_Logo(),
			new \Taxiar_Cta(),
			new \Taxiar_Gallery(),
			new \Taxiar_Info_Box(),
			new \taxiar_Portfolio(),
			new \taxiar_Portfolio_Info(),
			new \taxiar_Newsletter(),
			new \Taxiar_Menu(),

			new \Taxiar_Service_Tab(),
			new \Taxiar_Taxi_Service(),
			new \Taxiar_Taxi_Info(),
			new \taxiar_Download_Button(),
			new \Taxiar_Animated_Shape(),
			new \Taxiar_Arrows(),
			new \Taxiar_Tab_Builder(),
			new \taxiar_Skill(),
			new \Taxiar_About_Info(),
			new \taxiar_Step(),
			// new \taxiar_Choose(),
			new \Taxiar_Features(),
			new \taxiar_Video(),
			new \Taxiar_Price()
		];
	}

    public function widget_scripts() {

        wp_enqueue_script(
            'taxiar-frontend-script',
            TAXIAR_PLUGDIRURI . 'assets/js/taxiar-frontend.js',
            array('jquery'),
            false,
            true
		);

	}


	// public function taxiar_regsiter_widget_scripts( ) {

	// 	wp_register_script(
 //            'taxiar-tilt',
 //            TAXIAR_PLUGDIRURI . 'assets/js/tilt.jquery.min.js',
 //            array('jquery'),
 //            false,
 //            true
	// 	);
	// }


    function taxiar_elementor_widget_categories( $elements_manager ) {

        $elements_manager->add_category(
            'taxiar',
            [
                'title' => __( 'Taxiar', 'taxiar' ),
                'icon' 	=> 'fa fa-plug',
            ]
        );

        $elements_manager->add_category(
            'taxiar_footer_elements',
            [
                'title' => __( 'Taxiar Footer Elements', 'taxiar' ),
                'icon' 	=> 'fa fa-plug',
            ]
		);

		$elements_manager->add_category(
            'taxiar_header_elements',
            [
                'title' => __( 'Taxiar Header Elements', 'taxiar' ),
                'icon' 	=> 'fa fa-plug',
            ]
        );
	}
}

Taxiar_Extension::instance();